/*
   This is NOT the right way to get an OpenGL program to compile.

   Below is a bunch of stuff copied from the header glext.h.

   Just enough was copied in order to get our sample program to compile.

   Included in this file is an explanation of the runtime tricks that
   are being used here.
*/
#ifndef EDUCATIONAL_H
#define EDUCATIONAL_H

#if defined(_WIN32) && !defined(APIENTRY) && !defined(__CYGWIN__) && !defined(__SCITECH_SNAP__)
#define WIN32_LEAN_AND_MEAN 1
#include <windows.h>
#endif

#ifndef APIENTRY
#define APIENTRY
#endif
#ifndef APIENTRYP
#define APIENTRYP APIENTRY *
#endif


#ifndef GL_VERSION_2_0
/* GL type for program/shader text */
typedef char GLchar;       /* native character */
#endif

#ifndef GL_VERSION_1_5
/* GL types for handling large vertex buffer objects */
typedef ptrdiff_t GLintptr;
typedef ptrdiff_t GLsizeiptr;
#endif


/* Here are some constants that are not defined anywhere else. */
#define GL_ARRAY_BUFFER                   0x8892
#define GL_STATIC_DRAW                    0x88E4
#define GL_ELEMENT_ARRAY_BUFFER           0x8893
#define GL_FRAGMENT_SHADER                0x8B30
#define GL_VERTEX_SHADER                  0x8B31
#define GL_COMPILE_STATUS                 0x8B81
#define GL_LINK_STATUS                    0x8B82
#define GL_SHADING_LANGUAGE_VERSION       0x8B8C


/*
   In C, a function name is really a variable whose value is the
   address of the entry point into the function and the type of
   the variable is the signature of the function.

   A function prototype like this

      int myFunction(int x, int y);

   declares the variable myFunction, and gives that variable a
   type, int (*)(int,int). But the function prototype does NOT
   give the variable myFunction a value.

   The variable myFunction can acquire a value in a few different
   ways. If your program defines myFunction later in the code, then
   myFunction gets its value at compile time. If your code doesn't
   define myFunction, then myFunction can get its value at link time
   when the linker finds a definition for myFunction in some (binary)
   object file. It is also possible for myFunction to not get a value
   until runtime. By analogy, think of the following two lines of code.
      int x, y=5;
      x = y*y;
   The variable y gets a value at compile time, but the variable x does
   not get a value until runtime. The same kind of thing can be done
   with function names; they do not get a value until runtime.
   That is the situation we are in here.

   The new OpenGL functions that we need are defined in a DLL file.
   Rather than link to them in the "usual" way that you link to dll's,
   we will dynamically "load" the functions we need at runtime.

   The first block of code below defines the function types that we
   need in order to declare our function variables.

   The second block of code declares function variables, one for each of
   the new OpenGL functions that we need, and the declarations use the
   function types defined in the first block of code. Each function needs
   its own function type because almost all of the functions have different
   signatures.

   The third block of code is called at runtime and it assigns values to our
   function variables, These values are the entry points to the function
   definitions in the dll. Once these function variables have values, we can
   use the function variables to actually call the function definitions.
*/


/* Definitions of function types (taken from glext.h) */
typedef void (APIENTRYP PFNGLGENBUFFERSPROC) (GLsizei n, GLuint *buffers);
typedef void (APIENTRYP PFNGLBINDBUFFERPROC) (GLenum target, GLuint buffer);
typedef void (APIENTRYP PFNGLBUFFERDATAPROC) (GLenum target, GLsizeiptr size, const GLvoid *data, GLenum usage);
typedef void (APIENTRYP PFNGLBUFFERSUBDATAPROC) (GLenum target, GLintptr offset, GLsizeiptr size, const GLvoid *data);
typedef void (APIENTRYP PFNGLDELETEBUFFERSPROC) (GLsizei n, const GLuint *buffers);
typedef void (APIENTRYP PFNGLGETBUFFERPARAMETERIVPROC) (GLenum target, GLenum pname, GLint *params);
typedef GLvoid* (APIENTRYP PFNGLMAPBUFFERPROC) (GLenum target, GLenum access);
typedef GLboolean (APIENTRYP PFNGLUNMAPBUFFERPROC) (GLenum target);
typedef void (APIENTRYP PFNGLDRAWRANGEELEMENTSPROC) (GLenum mode, GLuint start, GLuint end, GLsizei count, GLenum type, const GLvoid *indices);
typedef void (APIENTRYP PFNGLSHADERSOURCEPROC) (GLuint shader, GLsizei count, const GLchar* *string, const GLint *length);
typedef void (APIENTRYP PFNGLCOMPILESHADERPROC) (GLuint shader);
typedef GLuint (APIENTRYP PFNGLCREATESHADERPROC) (GLenum type);
typedef void (APIENTRYP PFNGLGETSHADERIVPROC) (GLuint shader, GLenum pname, GLint *params);
typedef void (APIENTRYP PFNGLGETSHADERINFOLOGPROC) (GLuint shader, GLsizei bufSize, GLsizei *length, GLchar *infoLog);
typedef GLuint (APIENTRYP PFNGLCREATEPROGRAMPROC) (void);
typedef void (APIENTRYP PFNGLATTACHSHADERPROC) (GLuint program, GLuint shader);
typedef void (APIENTRYP PFNGLLINKPROGRAMPROC) (GLuint program);
typedef void (APIENTRYP PFNGLGETPROGRAMIVPROC) (GLuint program, GLenum pname, GLint *params);
typedef void (APIENTRYP PFNGLGETPROGRAMINFOLOGPROC) (GLuint program, GLsizei bufSize, GLsizei *length, GLchar *infoLog);
typedef GLint (APIENTRYP PFNGLGETUNIFORMLOCATIONPROC) (GLuint program, const GLchar *name);
typedef void (APIENTRYP PFNGLUSEPROGRAMPROC) (GLuint program);
typedef void (APIENTRYP PFNGLUNIFORM2FPROC) (GLint location, GLfloat v0, GLfloat v1);
typedef void (APIENTRYP PFNGLUNIFORM3FPROC) (GLint location, GLfloat v0, GLfloat v1, GLfloat v2);


/* Declarations of OpenGL function names using the above function types. */
PFNGLGENBUFFERSPROC              glGenBuffers;
PFNGLBINDBUFFERPROC              glBindBuffer;
PFNGLBUFFERDATAPROC              glBufferData;
PFNGLBUFFERSUBDATAPROC           glBufferSubData;
PFNGLDELETEBUFFERSPROC           glDeleteBuffers;
PFNGLGETBUFFERPARAMETERIVPROC    glGetBufferParameteriv;
PFNGLMAPBUFFERPROC               glMapBuffer;
PFNGLUNMAPBUFFERPROC             glUnmapBuffer;
PFNGLDRAWRANGEELEMENTSPROC       glDrawRangeElements;
PFNGLSHADERSOURCEPROC            glShaderSource;
PFNGLCOMPILESHADERPROC           glCompileShader;
PFNGLCREATESHADERPROC            glCreateShader;
PFNGLGETSHADERIVPROC             glGetShaderiv;
PFNGLGETSHADERINFOLOGPROC        glGetShaderInfoLog;
PFNGLCREATEPROGRAMPROC           glCreateProgram;
PFNGLATTACHSHADERPROC            glAttachShader;
PFNGLLINKPROGRAMPROC             glLinkProgram;
PFNGLGETPROGRAMIVPROC            glGetProgramiv;
PFNGLGETPROGRAMINFOLOGPROC       glGetProgramInfoLog;
PFNGLGETUNIFORMLOCATIONPROC      glGetUniformLocation;
PFNGLUSEPROGRAMPROC              glUseProgram;
PFNGLUNIFORM2FPROC               glUniform2f;
PFNGLUNIFORM3FPROC               glUniform3f;


/*
   At runtime(!), assign values to the function names.

   The Win32 function wglGetProcAddress() is what
   actually looks up the appropriate value for each
   function name.

   http://msdn.microsoft.com/en-us/library/dd374386(v=vs.85).aspx
*/
unsigned populate_functions(void)
{
   unsigned ok = 1;

   // Look up the function name n with type t.
   #define GB_GL(t,n)\
      ok &= (n = (t)wglGetProcAddress(#n)) ? 1 : printf("no "#n" found.\n") & 0;

   GB_GL(PFNGLGENBUFFERSPROC,glGenBuffers);
   GB_GL(PFNGLBINDBUFFERPROC,glBindBuffer);
   GB_GL(PFNGLBUFFERDATAPROC,glBufferData);
   GB_GL(PFNGLBUFFERSUBDATAPROC,glBufferSubData);
   GB_GL(PFNGLDELETEBUFFERSPROC,glDeleteBuffers);
   GB_GL(PFNGLGETBUFFERPARAMETERIVPROC,glGetBufferParameteriv);
   GB_GL(PFNGLMAPBUFFERPROC,glMapBuffer);
   GB_GL(PFNGLUNMAPBUFFERPROC,glUnmapBuffer);
   GB_GL(PFNGLDRAWRANGEELEMENTSPROC,glDrawRangeElements);
   GB_GL(PFNGLSHADERSOURCEPROC,glShaderSource);
   GB_GL(PFNGLCOMPILESHADERPROC,glCompileShader);
   GB_GL(PFNGLCREATESHADERPROC,glCreateShader);
   GB_GL(PFNGLGETSHADERIVPROC,glGetShaderiv);
   GB_GL(PFNGLGETSHADERINFOLOGPROC,glGetShaderInfoLog);
   GB_GL(PFNGLCREATEPROGRAMPROC,glCreateProgram);
   GB_GL(PFNGLATTACHSHADERPROC,glAttachShader);
   GB_GL(PFNGLLINKPROGRAMPROC,glLinkProgram);
   GB_GL(PFNGLGETPROGRAMIVPROC,glGetProgramiv);
   GB_GL(PFNGLGETUNIFORMLOCATIONPROC,glGetUniformLocation);
   GB_GL(PFNGLUSEPROGRAMPROC,glUseProgram);
   GB_GL(PFNGLUNIFORM2FPROC,glUniform2f);
   GB_GL(PFNGLUNIFORM3FPROC,glUniform3f);

   return ok;
}

#endif /* EDUCATIONAL_H */
