/*
    This program tests just what gcc means by "fuzzy-linking" when it
    implements its "stdcall-fixup" feature. In this file, test_function1()
    is declared cdecl (the default) and in the other file it is defined
    as stdcall. The reverse is true for test_function2(); it is declared
    stdcall here and defined as cdecl in the other file.

    If a cdecl function call is linked to stdcall function definition,
    then the stack is cleaned up twice. If a stdcall function call is
    linked to a cdecl function definition, then the stack doesn't get
    cleaned up at all. Either case could cause a crash.

    The link.exe and lcclnk.exe linkers from Visual Studio 6 and LCC-Win32,
    respectively, will not link the obj files from these two compilation units.
    But the ld linker from gcc will, using its "stdcall-fixup" (and ld issues
    several warnings).

    The cdecl function declaration test_function1() in this file is linked to
    a stdcall function definition in the other file. This causes the stack to
    be cleaned up twice, which "underflows" the stack. The call to printf()
    (just after the call test_function1(1,2,3,4)) corrupts the return address
    on the stack for the return to main().

    Notice that if test_function1() is redefined to take in three ints (instead
    of four) then the stack does not underflow and the program does not crash.
    Is gcc trying somehow to compensate, in some cases, for the mismatch between
    cdecl and stdcall? (See the assembly language listing to see the strange way
    gcc manipulates the stack pointer.)

    Here are the warnings produced by gcc (ld):
       Warning: resolving _test_function2@12 by linking to _test_function2
       Use --enable-stdcall-fixup to disable these warnings
       Use --disable-stdcall-fixup to disable these fixups
       Warning: resolving _test_function1 by linking to _test_function1@16
*/
#include <stdio.h>

extern int __cdecl   test_function1(int, int, int, int);
extern int __stdcall test_function2(int, int, int);

int z;

void extra_function(int w)
{
   z = test_function1(1, 2, 3, 4);  // stack cleaned up twice
   printf("1 + 2 = %d\n", z);

   z = test_function2(11, 12, 13);  // no stack clean up
   printf("11 - 12 = %d\n", z);

   return; // crashes when this tries to return
}

int main()
{
   int w = 101;
   extra_function(w);
   printf( "(w should be 101) w = %d", w );
   return 0;
}
