/*
    This program tests just what gcc means by "fuzzy-linking" when it
    implements its "stdcall-fixup" feature. In this file, test_function1()
    is declared cdecl (the default) and in the other file it is defined
    as stdcall. The reverse is true for test_function2(); it is declared
    stdcall here and defined as cdecl in the other file.

    If a cdecl function call is linked to stdcall function definition,
    then the stack is cleaned up twice. If a stdcall function call is
    linked to a cdecl function definition, then the stack doesn't get
    cleaned up at all. Either case could cause a crash.

    The link.exe and lcclnk.exe linkers from Visual Studio 6 and LCC-Win32,
    respectively, will not link the obj files from these two compilation units.
    But the ld linker from gcc will, using its "stdcall-fixup" (and ld issues
    several warnings).

    The stdcall function declaration test_function2() in this file is linked to
    a cdecl function definition in the other file. This causes the stack to
    not be cleaned. When the function call test_function2(11,12,13) is made
    over and over again, the lost space on the stack eventually causes the stack
    to reach the end of its allocated memory.

    This is not the only kind of problem that this mismatch in calling types
    can cause. But the other kinds of problems take more effort to demonstrate
    (i.e., registers pushed onto the stack do not get restored properly).

    Here are the warnings produced by gcc (ld):
       Warning: resolving _test_function2@4 by linking to _test_function2
       Use --enable-stdcall-fixup to disable these warnings
       Use --disable-stdcall-fixup to disable these fixups
*/
#include <stdio.h>

extern int __stdcall test_function2(int);

int z;

int main()
{
   int i;
   // 521172 seems to be the smallest number that causes "Stack Overflow" exception
   // if we use the MinGw version of gcc. The results are different if we use Cygwin.
   for (i = 0; i < 521172; i++)
   {
      z = test_function2(11);  // no stack clean up
   }
   fprintf(stderr, "done\n");
   return 0;
}
