#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include "glut.h"
#include "Dekker1.h"


void installShaders(void); // in ShaderExampleCU2.c


//============== Vertex buffer object code ==================

// Client buffer holding 2D polygon vertices representing
// a 3x5 glyph representing the number 6.
GLfloat sixVerts[][2] =
{
  {3.0, 4.5},       // 0
  {2.5, 5.0},       // 1
  {0.5, 5.0},       // 2
  {0.0, 4.5},       // 3
  {0.0, 0.5},       // 4
  {0.5, 0.0},       // 5
  {2.5, 0.0},       // 6
  {3.0, 0.5},       // 7
  {3.0, 2.5},       // 8
  {2.5, 3.0},       // 9
  {0.875, 3.0},     // 10
  {0.875, 4.125},   // 11
  {2.125, 4.125},   // 12
  {2.125, 3.75},    // 13
  {3.0,   3.75},    // 14
  {2.125, 2.125},   // 15  start of hole
  {0.875, 2.125},   // 16
  {0.875, 0.875},   // 17
  {2.125, 0.875}    // 18
};


// Client buffer holding triangle strip indices.
GLushort sixIndices[] =
{
  14, 13,  0, 12,  1, 11,  2, 10,  3, 16,  4,
  17,  5, 18,  6, 15,  7, 15,  8, 16,  9, 10
};


// Draw the number six.
void drawSix(void)
{
   // Handles for buffer objects holding
   // vertices and triangle strip indices.
   static GLuint sixVertBuffer;
   static GLuint sixIndexBuffer;

   // On first call, create buffer objects and load them
   // with the glyph's vertices and triangle strip indices.
   static GLboolean firstTime = GL_TRUE;
   if (firstTime)
   {
      glGenBuffers(1, &sixVertBuffer);
      glBindBuffer(GL_ARRAY_BUFFER, sixVertBuffer);
      glBufferData(GL_ARRAY_BUFFER, sizeof(sixVerts),
                   sixVerts, GL_STATIC_DRAW);

      glGenBuffers(1, &sixIndexBuffer);
      glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, sixIndexBuffer);
      glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(sixIndices),
                   sixIndices, GL_STATIC_DRAW);

      firstTime = GL_FALSE;
   }


   // Enable vertex arrays stored by client.
   glEnableClientState(GL_VERTEX_ARRAY);
   //glEnableClientState(GL_INDEX_ARRAY);  // (not necessary)


   // Bind the current vertex and index buffer to
   // the buffers holding the glyph for a six.
   glBindBuffer(GL_ARRAY_BUFFER, sixVertBuffer);
   glVertexPointer(2, GL_FLOAT, 0, (GLvoid*) 0);
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, sixIndexBuffer);


   // Draw the glyph. Indices represent a triangle strip.
   const GLsizei numIndices = sizeof(sixIndices)/sizeof(GLushort);
   glDrawRangeElements(GL_TRIANGLE_STRIP, 0, numIndices-1, numIndices,
                       GL_UNSIGNED_SHORT, (GLvoid*) 0);
}



//================= GLUT code ========================

// GLUT callback called just before first display event
// and every time the window is resized.
void reshape(int w, int h)
{
   glViewport(0,0, w,h);
}


// GLUT callback called when a new "display event" needs
// to be processed. This is where we draw our scene.
void display(void)
{
   glClear(GL_COLOR_BUFFER_BIT);
   drawSix();
   glFlush();
}


// GLUT callback called when the user presses a key.
void keyboard(unsigned char key, int x, int y)
{
   const unsigned char ESC = 27;
   if (key == ESC) exit(0);  // an ungraceful way to terminate
}


int main(int argc, char *argv[])
{
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_RGBA | GLUT_SINGLE);
   glutInitWindowSize(500,500);
   glutInitWindowPosition(10,10);
   glutCreateWindow("six");
   glutReshapeFunc(reshape);
   glutDisplayFunc(display);
   glutKeyboardFunc(keyboard);

   printf( "OpenGL version  => %s\n", glGetString(GL_VERSION) );
   printf( "  GLSL version  => %s\n", glGetString(GL_SHADING_LANGUAGE_VERSION) );
   printf( "OpenGL renderer => %s\n", glGetString(GL_RENDERER) );
   printf( "OpenGL vendor   => %s\n", glGetString(GL_VENDOR) );
   printf( "\nOpenGL extensions => %s\n", glGetString(GL_EXTENSIONS) );

   if ( !populate_functions() )
   {
      printf("Some OpenGL extensions not supported.\n");
      exit(0);
   }

   installShaders();

   //glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
   glClearColor(0, 0, 0.3, 1);
   glutMainLoop();
   return 0;
}
